//---------------------------------------------------------------------
#property copyright 	"Dima S."
#property link      	"dimascub@mail.com"
#property version   	"1.00"
#property description "Test Expert Advisor."
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	Connected libraries:
//---------------------------------------------------------------------
#include	<Trade\Trade.mqh>
#include	<BalanceSlopeControl.mqh>
//---------------------------------------------------------------------

//=====================================================================
//	External parameters:
//=====================================================================

//---------------------------------------------------------------------
//	Parameters of the system of controlling the slope of the balance curve;
//---------------------------------------------------------------------
enum SetLogic 
{
	No = 0,
	Yes = 1,
};
//---------------------------------------------------------------------
input SetLogic		UseAutoBalanceControl = No;
//---------------------------------------------------------------------
input ControlType	BalanceControlType = STEP_WITHOUT_HYSTERESIS;
//---------------------------------------------------------------------
//	Number of last trades for calculation of linear regression of the balance curve:
input int					TradesNumberToCalcLR = 3;
//---------------------------------------------------------------------
//	Maximum slope of linear regression to decrease trade volume to the minimum:
input double			LRKoeffForRejectLots = -0.020;
//---------------------------------------------------------------------
//	Minimal slope of linear regression to restore the normal mode of trading:
input double			LRKoeffForRestoreLots = 0.005;
//---------------------------------------------------------------------
//	Slope of linear regression to work in the intermediate mode:
input double			LRKoeffForIntermedLots = -0.019;
//---------------------------------------------------------------------
//	Decrease the input volume to the specified value when linear regression is sloped down
input double			RejectedLots = 0.10;
//---------------------------------------------------------------------
//	Normal work volume in the MM mode of fixed volume:
input double			NormalLots = 1.0;
//---------------------------------------------------------------------
input int									CCIPeriod = 50;
input ENUM_APPLIED_PRICE	CCIPrice = PRICE_TYPICAL;
//---------------------------------------------------------------------

//---------------------------------------------------------------------
int			ts1_indicator_handle1 = 0;
//---------------------------------------------------------------------

TBalanceSlopeControl	BalanceControl;

//---------------------------------------------------------------------
//	Handle of the initialization event:
//---------------------------------------------------------------------
int
OnInit( )
{
//	Create handle of the external indicator for further using of it:
	ResetLastError( );
	ts1_indicator_handle1 = iCCI( Symbol( ), PERIOD_CURRENT, CCIPeriod, CCIPrice );

//	If the initialization fails, return non-zero code:
	if( ts1_indicator_handle1 == INVALID_HANDLE )
	{
		Print( "Failed to initialize the CCI indicator, Code = ", GetLastError( ));
		return( -1 );
	}

//	Adjust the system of controlling the slope of balance curve:
	BalanceControl.SetTradeSymbol( Symbol( ));
	BalanceControl.SetControlType( BalanceControlType );
	BalanceControl.SetControlParams( LRKoeffForRejectLots, LRKoeffForRestoreLots, LRKoeffForIntermedLots );
	BalanceControl.SetSlopePoints( TradesNumberToCalcLR );
	BalanceControl.SetFiltrParams( 0, -1, 0 );
	BalanceControl.SetMonitoringBeginDate( 0 );

	return( 0 );
}

//---------------------------------------------------------------------
//	Handle of deinitialization event:
//---------------------------------------------------------------------
void
OnDeinit( const int _reason )
{
//	Delete handle of the indicator:
	if( ts1_indicator_handle1 != INVALID_HANDLE )
	{
		IndicatorRelease( ts1_indicator_handle1 );
	}
}

//---------------------------------------------------------------------
//	Handle of the event of coming of new tick by the current symbol:
//---------------------------------------------------------------------
int			current_signal = 0;
double	current_lots;
//---------------------------------------------------------------------
void
OnTick( )
{
//	Wait for the beginning of new bar:
	if( CheakNewBar( ) != 1 )
	{
		return;
	}

//	Get signal to open/close position:
	current_signal = GetSignal( 1 );

//	Select position by the current symbol:
	if( PositionSelect( Symbol( )) == true )
	{
//	Check if we need to close the opposite position:
		if( CheakPositionClose( current_signal ) == 1 )
		{
			return;
		}
	}

//	Update market information:
	BalanceControl.RefreshSymbolInfo( );

//	Check the presence of BUY signal:
	if( CheckBuySignal( current_signal ) == 1 )
	{
		if( UseAutoBalanceControl == Yes )
		{
			current_lots = BalanceControl.CalcTradeLots( RejectedLots, NormalLots );
		}
		else
		{
			current_lots = NormalLots;
		}
		CTrade	trade;
		trade.PositionOpen( Symbol( ), ORDER_TYPE_BUY, current_lots, SymbolInfoDouble( Symbol( ), SYMBOL_ASK ), 0, 0 );
	}

//	Check the presence of SELL signal:
	if( CheckSellSignal( current_signal ) == 1 )
	{
		if( UseAutoBalanceControl == Yes )
		{
			current_lots = BalanceControl.CalcTradeLots( RejectedLots, NormalLots );
		}
		else
		{
			current_lots = NormalLots;
		}
		CTrade	trade;
		trade.PositionOpen( Symbol( ), ORDER_TYPE_SELL, current_lots, SymbolInfoDouble( Symbol( ), SYMBOL_BID ), 0, 0 );
	}
}

//---------------------------------------------------------------------
//	Check if we need to close the position:
//---------------------------------------------------------------------
//	returns:
//		0 - no open position;
//		1 - position is already opened in direction of the signal;
//---------------------------------------------------------------------
int
CheakPositionClose( int _signal )
{
	long		position_type = PositionGetInteger( POSITION_TYPE );

	if( _signal == 1 )
	{
//	If a BUY position is already opened, return:
		if( position_type == ( long )POSITION_TYPE_BUY )
		{
			return( 1 );
		}
	}

	if( _signal == -1 )
	{
//	If a SELL position is already opened, return:
		if( position_type == ( long )POSITION_TYPE_SELL )
		{
			return( 1 );
		}
	}

//	Closing position:
	CTrade	trade;
	trade.PositionClose( Symbol( ), 10 );

	return( 0 );
}

//---------------------------------------------------------------------
//	Check if there is the Buy signal:
//---------------------------------------------------------------------
//	returns:
//		0 - no signal;
//		1 - there is the BUY signal;
//---------------------------------------------------------------------
int
CheckBuySignal( int _curr_signal )
{
	if( _curr_signal == 1 )
	{
		return( 1 );
	}

	return( 0 );
}

//---------------------------------------------------------------------
//	Check if there is the SELL signal:
//---------------------------------------------------------------------
//	returns:
//		0 - no signal;
//		1 - there is the SELL signal;
//---------------------------------------------------------------------
int
CheckSellSignal( int _curr_signal )
{
	if( _curr_signal == -1 )
	{
		return( 1 );
	}

	return( 0 );
}

//---------------------------------------------------------------------
//	Get signal for opening/closing position:
//---------------------------------------------------------------------
int
GetSignal( int _index )
{
	double		cci_array[ 2 ];

//	Get value of CCI:
	ResetLastError( );
	if( CopyBuffer( ts1_indicator_handle1, 0, _index, 2, cci_array ) != 2 )
	{
		Print( "Error of copying CopyBuffer, Code = ", GetLastError( ));
		return( 0 );
	}

	if( cci_array[ 1 ] >= 45.0 && cci_array[ 0 ] < 0.0 )
	{
		return( 1 );
	}
	else if( cci_array[ 1 ] <= 45.0 && cci_array[ 0 ] > 0.0 )
	{
		return( -1 );
	}

	return( 0 );
}

//---------------------------------------------------------------------
//	Returns a sign of appearance of a new bar:
//---------------------------------------------------------------------
//	- if it returns 1, there is a new bar;
//---------------------------------------------------------------------
int
CheakNewBar( )
{
	MqlRates		current_rates[ 1 ];

	ResetLastError( );
	if( CopyRates( Symbol( ), Period( ), 0, 1, current_rates ) != 1 )
	{
		Print( "Error of copying CopyRates, Code = ", GetLastError( ));
		return( 0 );
	}

	if( current_rates[ 0 ].tick_volume > 1 )
	{
		return( 0 );
	}

	return( 1 );
}
